package com.sillycycle.bagleyd.panex;

import android.app.Activity;
import android.content.Context;
//import android.content.res.Resources;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View.OnFocusChangeListener;
import android.view.View.OnKeyListener;
import android.widget.Button;
import android.widget.EditText;

import com.sillycycle.bagleyd.panex.model.PanexFormat;
import com.sillycycle.bagleyd.panex.model.PanexFormat.Modes;
import com.sillycycle.bagleyd.panex.view.PanexView;

/**
 * Android UI demo program
 */

public class PanexApp extends Activity {
	static PanexView aView = null;
	static EditText editText = null;
	static MediaPlayer bumpPlayer = null;
	static MediaPlayer movePlayer = null;
	static MediaPlayer dripPlayer = null;
	static final float VOLUME = (float) 0.4;
	static final boolean DEBUG = false; // noisy logs

	/** Listen for taps. */
	private static final class TrackingTouchListener
			implements View.OnTouchListener {
		PanexApp aApp;
		TrackingTouchListener(PanexApp app) {
			aApp = app;
		}

		@Override public boolean onTouch(View v, MotionEvent event) {
			int action = event.getAction();
			switch (action & MotionEvent.ACTION_MASK) {
			case MotionEvent.ACTION_DOWN:
			case MotionEvent.ACTION_POINTER_DOWN:
				if (PanexView.getPanexCanvas() == null) {
					return false;
				}
				PanexView.getPanexCanvas().selectPuzzle((int) event.getX()/*, (int) event.getY()*/);
				//aView.invalidate();
				break;
			case MotionEvent.ACTION_UP:
			case MotionEvent.ACTION_POINTER_UP:
			//case MotionEvent.ACTION_MOVE:
				if (PanexView.getPanexCanvas() == null) {
					return false;
				}
				boolean jump = PanexView.getPanexCanvas().jumpMove(PanexView.getPanexCanvas().currentStack,
						PanexView.getPanexCanvas().findTile((int) event.getX()));
				int inc = 0;
				if (PanexView.getPanexCanvas().releasePuzzle((int) event.getX()/*, (int) event.getY()*/)) {
					inc = 1;
					editText = (EditText) aApp.findViewById(R.id.text1);
					//editText.setText(changeToDecimal(PanexApp.digits));
					moves += inc;
					//editText.setText(changeToDecimal(digits));
					editText.setText(Integer.toString(moves));
					if (jump)
						aApp.playDrip();
					else
						aApp.playMove();
				}
				aView.invalidate();
				break;
			default:
				return false;
			}
			return true;
		}
	}

	/** The application model */

	/** The application view */
	private static int mode = PanexFormat.defaultMode.ordinal();
	private static int moves = 0;

	/** Called when the activity is first created. */
	@Override public void onCreate(Bundle state) {
		super.onCreate(state);

		// install the view
		setContentView(R.layout.standard);

		// find the panex view
		aView = (PanexView) findViewById(R.id.panex);

		aView.setOnCreateContextMenuListener(this);
		aView.setOnTouchListener(new TrackingTouchListener(this));

		aView.setOnKeyListener(new OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (KeyEvent.ACTION_DOWN != event.getAction()) {
					return false;
				}
				switch (keyCode) {
				case KeyEvent.KEYCODE_SPACE:
					//  color = Color.MAGENTA;
					break;
				case KeyEvent.KEYCODE_ENTER:
					//  color = Color.BLUE;
					break;
				default:
					return false;
				}
				//panexCanvas.processKey((char)event.getUnicodeChar()
				//	, false /* Control*/);
				return true;
			}
		});

		// Old way: moveTilesByPos -> ACTION_IGNORE -> updates display
		editText = (EditText) findViewById(R.id.text1);
		aView.setOnFocusChangeListener(new OnFocusChangeListener() {
			@Override public void onFocusChange(View v, boolean hasFocus) {
				PanexView.focusReact(hasFocus);
				Button buttonText = (Button) findViewById(R.id.button1);
				buttonText.setText(PanexFormat.titleName(PanexFormat.Modes.values()[mode].toString()));
				editText.setText(Integer.toString(moves));
			}
		});

		// wire up the controller
		(findViewById(R.id.button1)).setOnClickListener(
				new Button.OnClickListener() {
			@Override public void onClick(View v) {
				mode++;
				if (mode < Modes.HANOI.ordinal() ||
						mode > Modes.PANEX.ordinal())
					mode = 0;
				changeFormatPanex(mode);
				Button buttonText = (Button) findViewById(R.id.button1);
				buttonText.setText(PanexFormat.titleName(PanexFormat.Modes.values()[mode].toString()));
				moves = 0;
				editText.setText(Integer.toString(moves));
				playDrip();
				aView.invalidate();
			}
		});
		findViewById(R.id.menu_clear).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						clearPanex();
    					}
				});
		findViewById(R.id.menu_increment).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						incrementPanex();
					}
				});
		findViewById(R.id.menu_undo).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						undoPanex();
					}
				});
		findViewById(R.id.menu_solve).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						solvePanex();
					}
				});
		findViewById(R.id.menu_decrement).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						decrementPanex();
					}
				});
		findViewById(R.id.menu_redo).setOnClickListener(
				new Button.OnClickListener() {
					@Override public void onClick(View v) {
						redoPanex();
					}
				});

		if (aView == null)
			return;
		aView.setTextChangeListener(new PanexView.TextChangeListener() {
			@Override public void onTextChange(String string) {
				moves++;
				//editText.setText(changeToDecimal(digits));
				editText.setText(Integer.toString(moves));
				playMove();
				aView.invalidate();
			}
		});
	}

	/** Install an options menu. */
	@Override public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.menu, menu);
		return true;
	}

	static void clearPanex() {
		if (aView == null)
			return;
		aView.clearPanex();
		moves = 0;
		editText.setText(Integer.toString(moves));
	}

	static void decrementPanex() {
		if (aView == null)
			return;
		aView.decrementPanex();
		moves = 0;
		editText.setText(Integer.toString(moves));
	}

	static void incrementPanex() {
		if (aView == null)
			return;
		aView.incrementPanex();
		moves = 0;
		editText.setText(Integer.toString(moves));
	}

	static void undoPanex() {
		if (aView == null)
			return;
		moves = aView.undoPanex();
		aView.invalidate();
		editText.setText(Integer.toString(moves));
	}

	static void redoPanex() {
		if (aView == null)
			return;
		moves = aView.redoPanex();
		aView.invalidate();
		editText.setText(Integer.toString(moves));
	}

	static void solvePanex() {
		if (aView == null)
			return;
		moves = aView.solvePanex();
		editText.setText(Integer.toString(moves));

	}

	private static void changeFormatPanex(int mode) {
		PanexView.changeFormatPanex(mode);
	}

	/*private void playBump() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("bump", "raw", context.getPackageName());

		if (bumpPlayer == null) {
			bumpPlayer = MediaPlayer.create(context, resID);
			bumpPlayer.setVolume(VOLUME, VOLUME);
		} else {
			bumpPlayer.pause();
		}
		//bumpPlayer.release();
		bumpPlayer.start();
	}*/

	private void playMove() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("move", "raw", context.getPackageName());

		if (movePlayer == null) {
			movePlayer = MediaPlayer.create(context, resID);
			movePlayer.setVolume(VOLUME, VOLUME);
		} else {
			movePlayer.pause();
		}
		//movePlayer.release();
		movePlayer.start();
	}

	private void playDrip() {
		if (DEBUG)
			return;
		Context context = aView.getContext();
		int resID = getResources().getIdentifier("drip", "raw", context.getPackageName());

		if (dripPlayer == null) {
			dripPlayer = MediaPlayer.create(context, resID);
			dripPlayer.setVolume(VOLUME, VOLUME);
		} else {
			dripPlayer.pause();
		}
		//dripPlayer.release();
		dripPlayer.start();
	}
	/** Respond to an options menu selection. */
	@Override public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_clear:
			clearPanex();
			return true;
		case R.id.menu_decrement:
			decrementPanex();
			return true;
		case R.id.menu_increment:
			incrementPanex();
			return true;
		case R.id.menu_undo:
			undoPanex();
			return true;
		case R.id.menu_redo:
			redoPanex();
			return true;
		case R.id.menu_solve:
			solvePanex();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	/** Install a context menu. */
	@Override public void onCreateContextMenu(
			ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		menu.add(Menu.NONE, 1, Menu.NONE, "Clear")
			.setAlphabeticShortcut('c');
		menu.add(Menu.NONE, 1, Menu.NONE, "Decrement")
			.setAlphabeticShortcut('d');
		menu.add(Menu.NONE, 1, Menu.NONE, "Increment")
			.setAlphabeticShortcut('i');
		menu.add(Menu.NONE, 1, Menu.NONE, "Undo")
			.setAlphabeticShortcut('u');
		menu.add(Menu.NONE, 1, Menu.NONE, "Redo")
			.setAlphabeticShortcut('r');
		menu.add(Menu.NONE, 1, Menu.NONE, "Solve")
			.setAlphabeticShortcut('s');
	}

	/** Respond to a context menu selection. */
	@Override public boolean onContextItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case 1:
			clearPanex();
			return true;
		case 2:
			decrementPanex();
			return true;
		case 3:
			incrementPanex();
			return true;
		case 4:
			undoPanex();
			return true;
		case 5:
			redoPanex();
			return true;
		case 6:
			solvePanex();
			return true;
		default:
		}
		return false;
	}
}
